local addonName, localization = ...
local L = nil

local addonColor = "|cffff4488"
local abilityColor = "|cffeda55f"
local targetColor = "|cffff0000"
local amountColor = "|cff00dbba"
local prefix = ("%s%s|r:"):format(addonColor, addonName)
local HELP = nil

local filter = 0x001

local addon = CreateFrame("Frame")
addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)

-- isCrit, amount, abilityName
local function spellDamage(_, ability, _, amount, _, _, _, _, _, isCrit) return isCrit, amount, ability end
local function swingDamage(amount, _, _, _, _, _, isCrit) return isCrit, amount, L["Melee"] end
local function spellHeal(_, ability, _, amount, _, _, isCrit) return isCrit, amount, ability end

local events = {
	SPELL_DAMAGE = spellDamage,
	SPELL_HEAL = spellHeal,
	SWING_DAMAGE = swingDamage, 
}

function addon:COMBAT_LOG_EVENT_UNFILTERED(_, eventType, _, _, srcFlags, _, dstName, _, ...)
	if not events[eventType] then return end
	if bit.band(srcFlags, filter) == 0 then return end
	local isCrit, amount, abilityName = events[eventType](...)
	if not isCrit or not abilityName or type(amount) ~= "number" then return end
	if not CritSpitDB[abilityName] or CritSpitDB[abilityName] < amount then
		CritSpitDB[abilityName] = amount
		print(L["%s %s%s|r crit on %s%s|r for %s%d|r."]:format(prefix, abilityColor, abilityName, targetColor, dstName, amountColor, amount))
	end
end

function addon:ADDON_LOADED(msg)
	if msg == "Blizzard_CombatLog" then
		filter = COMBATLOG_OBJECT_AFFILIATION_MINE
	elseif msg == addonName then
		if not CritSpitDB then CritSpitDB = {} end
		L = localization:Init()
		HELP = {
			L["/critspit all: Prints all your highest critical hits to the local chat frame."],
			L["/critspit <ability name>: Prints the highest crit with the given ability to your local chat frame."],
			L["/critspit p/g/r/s <ability name>: Prints the highest crit to party, guild, raid chat or say."],
			L["/critspit wipe: Wipes saved data."],
			L["/critspit wipe <ability name>: Wipes the saved data for the given ability."],
		}
	end
end

addon:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
addon:RegisterEvent("ADDON_LOADED")

local channels = {
	p = "PARTY",
	g = "GUILD",
	r = "RAID",
	s = "SAY",
}
local function printHelp()
	for i, v in next, HELP do
		print(v)
	end
end

SlashCmdList.CritSpit = function(input)
	input = input:trim()
	if input:len() == 0 then
		printHelp()
	elseif input == "all" then
		for k, v in pairs(CritSpitDB) do
			print(L["%s %s%s|r crit for %s%d|r."]:format(prefix, abilityColor, k, amountColor, v))
		end
	elseif input:find("^wipe") then
		local what = select(3, input:find("^wipe%s(.*)"))
		if type(what) == "nil" then
			CritSpitDB = {}
			print(L["%s All saved critical hits have been wiped."]:format(prefix))
		elseif CritSpitDB[what] then
			CritSpitDB[what] = nil
			print(L["%s Wiped saved critical hit record for %s%s|r."]:format(prefix, abilityColor, what))
		else
			printHelp()
		end
	elseif input:find("^%l%s") then
		local channel, ability = select(3, input:find("^(%l)%s(.*)"))
		if not channels[channel] then return end
		if not CritSpitDB[ability] then return end
		SendChatMessage(L["CritSpit Record: %s crit for %d."]:format(ability, CritSpitDB[ability]), channels[channel])
	elseif CritSpitDB[input] then
		print(L["%s %s%s|r crit for %s%d|r."]:format(prefix, abilityColor, input, amountColor, CritSpitDB[input]))
	end
end
SLASH_CritSpit1 = "/critspit"

if LibStub then
	local ldb = LibStub:GetLibrary("LibDataBroker-1.1", true)
	if ldb then
		local csLDB = ldb:NewDataObject(addonName, {
			type = "data source",
			text = addonName,
			icon = "Interface\\Icons\\Spell_Shadow_AbominationExplosion",
		})
		function csLDB.OnTooltipShow(tt)
			tt:AddLine(addonName)
			for k, v in pairs(CritSpitDB) do
				tt:AddDoubleLine(k, v, 1, 1, 1, 1, 0, 0)
			end
			tt:AddLine(L["Use %s/critspit|r to output critical hits in channels."]:format(addonColor), 0.2, 1, 0.2, 1)
		end
	end
end

